#include <stdio.h>
#include <stdlib.h>

#include "config.h"
#include "gif_lib.h"

static int iOffset[]={0,4,2,1};
static int iJumps[] ={8,8,4,2};

/*     Gif 
*/
int
_writeImage(GifFileType *gf, image *img)
{ GifColorType cmap[2]={{255,255,255},{0,0,0}};
  unsigned y;
  if(!gf)
   return 1;
  if(EGifPutScreenDesc(gf,img->width,img->height,2,0,1,cmap)==GIF_ERROR)
   return 2;
  if(EGifPutImageDesc(gf,0,0,img->width,img->height,0,1,NULL)==GIF_ERROR)
   return 3;
  for(y=0; y!=img->height; y++)
   if(EGifPutLine(gf,img->rows[y],img->width)==GIF_ERROR)
    return 4;
  if(EGifCloseFile(gf)==GIF_ERROR)
   return 5;
  return 0;
}

/*      
*/
int
writeImage(char *name, image *img)
{ return _writeImage(EGifOpenFileName(name,0),img);
}

/*      
*/
int
writeImageToHandle(int fd, image *img)
{ return _writeImage(EGifOpenFileHandle(fd),img);
}

/*    
*/
int
putPixel(image *img, unsigned x, unsigned y, unsigned color)
{ if(!img)
   return 1;
  if(y>=img->height)
   { int i;
     if(!img->width)
      img->width=x+1;
     if(!img->rows)
      { if((img->rows=malloc(sizeof(char*)*(y+1)))==NULL)
         return 4;
      }
     else
      { if((img->rows=realloc(img->rows,sizeof(char*)*(y+1)))==NULL)
         return 5;
      }
     for(i=img->height; i!=y+1; i++)
      { unsigned j;
        if((img->rows[i]=malloc(sizeof(char)*img->width))==NULL)
         return 2;
        for(j=0; j!=img->width; j++)
         img->rows[i][j]=0;
      }
     img->height=y+1;
   }
  if(x>=img->width)
   { int i;
     for(i=0; i!=img->height; i++)
      { unsigned j;
        if((img->rows[i]=realloc(img->rows[i],sizeof(char)*(x+1)))==NULL)
         return 3;
        for(j=img->width; j!=x+1; j++)
         img->rows[i][j]=0;
      }
     img->width=x+1;
   }
  img->rows[y][x]=color;
  return 0;
}

int
insertImage(image *dst, const unsigned x, const unsigned y, const image *src, int over)
{ if(src->rows)
   { unsigned yy;
     for(yy=0; yy!=src->height; yy++)
      { unsigned xx;
        for(xx=0; xx!=src->width; xx++)
         { int rc=0;
           if(over || (rc=src->rows[yy][xx])!=0)
            rc=putPixel(dst,x+xx,y+yy,rc);
           if(rc)
            return rc;
         }
      }
   }
  return 0;
}

int
readImage(image *img, char *name)
{ GifFileType *gf=DGifOpenFileName(name);
  GifRecordType gr;
  GifColorType *cmap;
  unsigned cnum;

  if(!gf)
   return 1;
  if(putPixel(img,gf->SWidth-1,gf->SHeight-1,0))
   return 4;
  do
   { if(DGifGetRecordType(gf,&gr)==GIF_ERROR)
      return 2;
     switch(gr)
      { case IMAGE_DESC_RECORD_TYPE:
         if(DGifGetImageDesc(gf)==GIF_ERROR)
          return 3;
         if(gf->ITop+gf->IHeight>gf->SHeight ||
            gf->ILeft+gf->IWidth>gf->SWidth)
          return 5;
         if(gf->IInterlace)
          { int i;
            for(i=0; i!=4; i++)
             { int j;
               for(j=gf->ITop+iOffset[i];
                   j<gf->ITop+gf->IHeight;
                   j+=iJumps[i])
                if(DGifGetLine(gf,img->rows[j]+gf->ILeft,gf->IWidth)==GIF_ERROR)
                 return 6;
             }
          }
         else
          { unsigned i;
            for(i=0; i!=gf->IHeight; i++)
             if(DGifGetLine(gf,img->rows[i]+gf->ILeft,gf->IWidth)==GIF_ERROR)
              return 7;
          }
         break;
        case EXTENSION_RECORD_TYPE:
          { GifByteType *ext=NULL;
            int ecode;
            if(DGifGetExtension(gf,&ecode,&ext)==GIF_ERROR)
             return 8;
            while(ext)
             if(DGifGetExtensionNext(gf,&ext)==GIF_ERROR)
              return 9;
          }
         break;
        default:
         break;
      }
   } while(gr!=TERMINATE_RECORD_TYPE);
  cmap=gf->IColorMap ? gf->IColorMap : gf->SColorMap;
  cnum=gf->IColorMap ? gf->IBitsPerPixel : gf->SBitsPerPixel;
  cnum=1<<cnum;
   { int i;
     for(i=0; i!=cnum; i++)
      cmap[i].Red=(cmap[i].Red+cmap[i].Green+cmap[i].Blue)>383 ? 0:1;
     for(i=0; i!=gf->SHeight; i++)
      { int j;
        for(j=0; j!=gf->SWidth; j++)
         img->rows[i][j]=cmap[img->rows[i][j]].Red;
      }
   }
  if(DGifCloseFile(gf)==GIF_ERROR)
   return 10;
  return 0;
}

/*     
*/
int
clearImage(image *img)
{ int i;
  for(i=0; i!=img->height; i++) free(img->rows[i]);
  free(img->rows);
  img->rows=NULL;
  return img->height=img->width=0;
}
