#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "config.h"

int text2image(image *img,FILE *f,char *dir);

void
usage()
{ fputs("-- Text to gif converter. Copyright (c) 1995 Andrew Maltsev, <am@f1.ru>\n\n"
        "Usage: t2gif [-d dir] [-o outfile] [-f font] [-x] [file]\n"
        "\t-d\tuse dir for included files\n"
        "\t-o\twrite results into outfile (default is stdout)\n"
        "\t-f\tuse char shapes from .bdffont (default is koi9x15.bdf)\n"
        "\t-x\texpand font twice (usually needed)\n"
        ,stderr);
  exit(1);
}

int
main(int ac, char *av[])
{ int ch;
  char *oname=NULL;
  char *gdir=NULL;
  char *fn=NULL;
  image img={0,0,NULL};
  int rc,dbl=0;

  while((ch=getopt(ac,av,"Hhd:o:f:x"))!=-1)
   switch(ch)
    { case 'd':
       gdir=optarg;
       break;
      case 'o':
       oname=optarg;
       break;
      case 'f':
       fn=optarg;
       break;
      case 'x':
       dbl=1;
       break;
      case 'H':
      case 'h':
      default:
       usage();
    }
  ac-=optind;
  av+=optind;

  if((rc=readFont(fn ? fn : "koi9x15.bdf",dbl))!=0)
   { printf("Error reading font (rc=%d)\n",rc);
     return 1;
   }
  if((rc=text2image(&img,ac ? fopen(*av,"r") : stdin,gdir))!=0)
   { printf("Error making image (rc=%d)\n",rc);
     return 1;
   }
  printf("Image produced (%dx%d), writing..\n",img.width,img.height);
  if(oname)
   rc=writeImage(oname,&img);
  else
   rc=writeImageToHandle(1,&img);
  if(rc)
   { printf("Error writing image (rc=%d)\n",rc);
     return 1;
   }
  return 0;
}

int
text2image(image *img,FILE *f,char *dir)
{ unsigned y=0;
  unsigned w=font['M'].width;
  unsigned h=font['M'].height;
  char buf[100];

  if(!f) return 1;
  while(fgets(buf,100,f))
   { unsigned char *s;
     unsigned maxh=0;
     unsigned x=0;

     for(s=buf; *s!='\n' && *s; s++)
      { switch(*s)
         { case '\t':
            x=(x/w/8+1)*w*8;
            break;
           case '\r':
            x=0;
            break;
           case '\b':
            x-=w;
            break;
           case ' ':
            x+=w;
            break;
           case '{':
            if(*++s!='{')
             { char *ss=index(s,'}');
               char iname[1000];
               image ti={0,0,NULL};
               unsigned xx,yy;
               if(!ss)
                { fprintf(stderr,"No delimiter found (x=%d, y=%d)\n"
                                ,x/w,y/h);
                  break;
                }
               *ss=0;
               if(dir)
                sprintf(iname,"%s/%s",dir,s);
               else
                strcpy(iname,s);
               s=ss;
               if(readImage(&ti,iname))
                { fprintf(stderr,"Cannot read image `%s'\n",iname);
                  break;
                }
               xx=x>ti.width/2 ? x-ti.width/2 : 0;
               yy=y+h/2>ti.height/2 ? y+h/2-ti.height/2 : 0;
               if(insertImage(img,xx,yy,&ti,0))
                fprintf(stderr,"Cannot place image (%dx%d at %d,%d)\n"
                              ,ti.width,ti.height,xx,yy);
               clearImage(&ti);
               break;
             }
           default:
            if(insertImage(img,x,y,font+(*s),0))
             fprintf(stderr,"Cannot place `%c' at %d,%d\n"
                           ,*s,x,y);
            x+=font[*s].width;
            if(maxh<font[*s].height)
             maxh=font[*s].height;
            break;
         }
      }
     y+=maxh;
   }
  fclose(f);
  return 0;
}
